@@ -20,11 +20,12 @@ package org.apache.zookeeper.test;
 
 import java.util.concurrent.CountDownLatch;
 
-import org.apache.log4j.Logger;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.zookeeper.ZooKeeper;
 import org.apache.zookeeper.test.AsyncOps.ACLCB;
-import org.apache.zookeeper.test.AsyncOps.ChildrenCB;
 import org.apache.zookeeper.test.AsyncOps.Children2CB;
+import org.apache.zookeeper.test.AsyncOps.ChildrenCB;
 import org.apache.zookeeper.test.AsyncOps.DataCB;
 import org.apache.zookeeper.test.AsyncOps.StatCB;
 import org.apache.zookeeper.test.AsyncOps.StringCB;
@@ -33,23 +34,17 @@ import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
 
-/**
- * Functional testing of asynchronous operations, both positive and negative
- * testing.
- *
- * This just scratches the surface, but exercises the basic async functionality.
- */
 public class AsyncOpsTest extends ClientBase {
-    private static final Logger LOG = Logger.getLogger(AsyncOpsTest.class);
+    private static final Logger LOG = LoggerFactory.getLogger(AsyncOpsTest.class);
 
     private ZooKeeper zk;
 
     @Before
     @Override
-    protected void setUp() throws Exception {
+    public void setUp() throws Exception {
         super.setUp();
 
-        LOG.info("STARTING " + getName());
+        LOG.info("Creating client " + getTestName());
 
         zk = createClient();
         zk.addAuthInfo("digest", "ben:passwd".getBytes());
@@ -57,14 +52,12 @@ public class AsyncOpsTest extends ClientBase {
 
     @After
     @Override
-    protected void tearDown() throws Exception {
+    public void tearDown() throws Exception {
         zk.close();
 
         super.tearDown();
 
         LOG.info("Test clients shutting down");
-
-        LOG.info("FINISHED " + getName());
     }
 
     @Test
